//SLAVE CONTROLLER!!
//Controls solar panel heat collection system
//Judas Gutenberg
//part of SOLAR CONTROLLER III:
//February 20, 2011: added this slave arduino for more analog inputs, an LCD display, and a bunch of EEPROM
//June 23, 2011:  added a menu system developed since late April. 
//menu system happens entirely on this Slave (using the LCD and an IR remote), 
//and then sends commands to Master in the same format as commands to Master
//sent via the serial terminal (previously, the only way to reconfigure the controller).
 
#include <LiquidCrystal.h>
#include <Wire.h>    
#include <EEPROM.h>
#include <IRremote.h>
#define  SLAVE  19     
 
byte value; 
int temp1;
int temp2;
int temp3;
int temp4;
int lasteepromread;
byte lastpinread;
char charblock[20][8];
unsigned int xpos[8];
unsigned int ypos[8];
byte extrabyte[3];

byte outcount=0;
byte low, high;
byte commandbyte=0;
byte readvalue;
unsigned int address;
 
byte * deliverable;
bool nonlcdmode=false;
byte slave_packet[18]; 
byte  commandpacket[18];
LiquidCrystal lcd(7, 8, 9, 10, 11, 12);
int RECV_PIN = 2;
IRrecv irrecv(RECV_PIN);
decode_results results;
long IRresval;
byte IRlow;
byte IRhigh;
byte ir_packet[18]; 
int integers[12]={255,32895,16575,49215,8415,41055,24735,57375,4335,36975,31371343,31357063};//last two are colon and dash
//commands are in this order: ??, up, down, left, right, enter(ok), delete, quit
int commands[8]={0,49725,8925,17085,33405,41565,39015,31347373};
bool menumode=false;
bool oldmenumode=false;
bool canmenu=false;

//default data takes this form (comma-delimited, no spaces). it is char data for use internally to initially represent ir menu states, populated by  master arduino initially and considered read-only:
//SECONDS,MINUTES,HOURS,DAYOFWEEK,DAY,MONTH,YEAR,SEASON,MAXHOTWATER,MINSUMMER,MINWINTER,CYCLETIME
char irinternaldata[]="00,15,15,1,4,5,11,1,155,120,100,122,22,22,22,22,22,22,182,"; 
//irinternaldata is mostly only used for debugging; it's the default values for the menu system
//but they are usually quickly overwritten with more valid values from the master 

extern int __bss_end;
extern int *__brkval;

//the first of these is for date, the second for time; have to do a little math later to keep these arrays small

const char *mastercommand[9] = 
{
	"ss",
	"sW",
	"sD",
	"sT",
	"sx",
	"sm",
	"sw",
	"st"
}
;

int  dataplace[4][3] = 
{ 
	 
	{7},
	{3},
	{6,5,4},
	{2,1,0},
	
}
;

const char *legend[2][3] = 
{
	{"YY","MM","DD"},
	{"HH","MM","SS"}
}
;
const char *menuitem[3][9] = 
{ 
	{   
		"Set season",
		"Day of week",
		"Change date",
		"Change time",
		"Set max hotwater",
		"Set min summer",
		"Set min winter",
		"Set cycle time",
		"*"
	},
	{
		"Winter",
		"Summer",
		"*"	
	},
	{
		"Sunday",
		"Monday",
		"Tuesday",
		"Wednesday",
		"Thursday",
		"Friday",
		"Saturday",
		"*"	
	}
}
;

//got to change if you increase number of menu items
int arrDataForMenu[12]; //contains int representations of data for display in menu.  
//basically the same data as irinternaldata exploded on comma and its items turned into type integer.
int lcdtop=8;  //got to change if you increase number of menu items

int lcdcursor=0;
int nowmenu=0;
byte ircursor=0;
byte currentlcdlimit=0;
bool linemode=false;
bool bwlDisableRightward=true;


byte ircursorlimit;


void setup() 
{ 
	commandpacket[0]='u';
	commandpacket[1]='s';
	Wire.begin(SLAVE); 
	Wire.onReceive(printstuff);
	Wire.onRequest(outgoing); 
	Serial.begin(57600);
	lasteepromread=0;
	lcd.begin(20, 4);
	irrecv.enableIRIn(); 
	populatedefaultarray();
 
} 

void loop () 
{ 
	//for(int i=0; i<10; i++)
	//{
		//Serial.println((int) arrDataForMenu[i]);
	//}
	byte j;
	byte k;
	byte thisbyte;
	long thispower;
	int analogvalue;
	int irval;
	
	//Serial.print((int) 1);
	if (irrecv.decode(&results)  && canmenu) 
	{
		IRresval=results.value;
		IRlow=IRresval % 256;
		IRhigh=(IRresval-IRlow)/256;
		//Serial.print((int) IRhigh);
		//Serial.print(" ");
		//Serial.print((int) IRlow);
		//Serial.println(" ");
		//Serial.println((unsigned int)IRresval);
		irval=ircodelookup(IRresval);
		//Serial.print("          ");
		//for testing new IR codes:
		Serial.println(IRresval);
		//Serial.print(":");
		//Serial.println(irval);
		if(!menumode)
		{
			bwlDisableRightward=true;
		}
		
		if((!linemode && irval==1)  && lcdcursor>0)//going up in a particular menu
		{
			menumode=true;
			lcdcursor--;
			ircursor=0;
			if(nowmenu>0  && nowmenu<3)
			{
				//Serial.println("menu datachange!");
				//Serial.print(nowmenu);
				//Serial.print(" ");
				//Serial.print(lcdcursor);
				IRchangeInternalData(nowmenu, lcdcursor);
			}
			else
			{
				bwlDisableRightward=false;
			}
		}
		currentlcdlimit=arraylimit(menuitem[nowmenu]);
		if(currentlcdlimit<2  || currentlcdlimit>20)
		{
			linemode=true;	
			bwlDisableRightward=false;
			
		}
		if((!linemode && irval==2 )  && lcdcursor<currentlcdlimit-1 )//going down in a particular menu
		{
			
			menumode=true;
			lcdcursor++;
			if(nowmenu>0  && nowmenu<3)
			{
				//Serial.println("menu datachange!");
				//Serial.print(nowmenu);
				//Serial.print(" ");
				//Serial.print(lcdcursor);
				IRchangeInternalData(nowmenu, lcdcursor);
			}
		 	else
			{
				bwlDisableRightward=false;
			}
		}
		//4 is right and 2 is down
	 
		if(!linemode && irval==4  && !bwlDisableRightward || linemode && irval==2)//going into a new menu or up out of linemode
		{
			if(menumode)
			{
				nowmenu=lcdcursor+1;
			}
			menumode=true;
			if(nowmenu==3  || nowmenu==4)//dates or times
			{
				ircursorlimit=5;
				
			}

			else //usually setting temperatures, which have three digits
			{
				ircursorlimit=2;	
			}
			if(nowmenu<3  && nowmenu>0) //changing season or day of week, so need to set a default from the known data in the menu
			{
				Serial.print("default val:");
				Serial.println(arrDataForMenu[dataplace[nowmenu-1][0]]);
				lcdcursor=arrDataForMenu[dataplace[nowmenu-1][0]];
				bwlDisableRightward=true;
			}
			else
			{
				bwlDisableRightward=false;
			}
			
		}
		if(!linemode && irval==3  || linemode && irval==1 ) //going out of a menu to a higher level; 3 is left 1 is up
		{
			menumode=true;
			nowmenu=0;
			linemode=false;
			ircursor=0;
			bwlDisableRightward=false;
		 
		}
		if(linemode && irval==3  && ircursor>0)//3 is cursor left  
		{
			
			ircursor--;
		}
		if(linemode && irval==4  && ircursor<ircursorlimit)//4 is cursor right
		{
			ircursor++;
		}
		if(irval>47)
		{
			menumode=true;
			IRchangeInternalData( nowmenu,  irval);
			if(ircursor<ircursorlimit)
			{
				ircursor++;
			}
			
		}
		else
		{
			//ircursor=0;	
		}
		if(irval==5)//prepare command for Master to pick up
		{
			
			byte i;
			byte commandpacketcursor=0;
			byte j, thisbyte;
			bool bwlHadNonZero=false;
			int thispower;
			int thisnumber;
			for(i=0; i<2; i++)
			{
				commandpacket[commandpacketcursor]=mastercommand[nowmenu-1][i];
				commandpacketcursor++;
			}
			commandpacket[commandpacketcursor]=' ';
			commandpacketcursor++;
			
			if(nowmenu<3) 
			{
				thisnumber= 48 + arrDataForMenu[dataplace[nowmenu-1][0]];
				commandpacket[commandpacketcursor]=thisnumber;
				commandpacketcursor++;
			}
		 	else if(nowmenu<5   )  //delimited data such as time and date
			{
				for(i=0; i<3; i++)
				{
					thisnumber=arrDataForMenu[dataplace[nowmenu-1][i]];
					//working on it!!
					

					for(j=0; j<2; j++)
					{
						thispower=powerof(10, (1-j));
						thisbyte=thisnumber/(thispower);
				 		
						thisnumber=thisnumber-(thisbyte * (thispower));
						commandpacket[commandpacketcursor]=thisbyte +48;
						commandpacketcursor++;
					}
					commandpacket[commandpacketcursor]=':'; //spaces weren't working
					commandpacketcursor++;
					
				}
				
			}
			else if(nowmenu>4) //for temperatures, we don't store the menu position locations
			{
				thisnumber=arrDataForMenu[nowmenu+3];
				Serial.println((int)thisnumber);
				for(j=0; j<4; j++)
				{
					thispower=powerof(10, (3-j));
					thisbyte=thisnumber/(thispower);
					if(!bwlHadNonZero)
					{
						bwlHadNonZero=thisbyte>0;
					}
					//Serial.println((int)thisbyte);
			 	 
					thisnumber=thisnumber-(thisbyte * (thispower));
					if(thisbyte!=0 || bwlHadNonZero)
					{
						commandpacket[commandpacketcursor]=thisbyte +48;
						//Serial.println(thisbyte +48);
						commandpacketcursor++;
					}
				}
			}
			commandpacket[commandpacketcursor]=0;
			//Serial.println( commandpacket);
			ClearLCD();
			lcd.setCursor(0, 1);
			lcd.print("Changes saved.");
			delay(200);
	 		menumode=false;
	 		irval=0;
	 		lcdcursor=0;
			ircursor=0;
			nowmenu=0;
			linemode=false;
		}
		else if (irval==7)//quit menu mode and tell master to repopulate defaults
		{
			ClearCommandPacket();
			commandpacket[0]='u'; 
			commandpacket[1]='s';
			//Serial.println( commandpacket);
			ClearLCD();
			lcd.setCursor(0, 1);
			lcd.print("Changes abandoned.");
			delay(200);
	 		menumode=false;
	 		irval=0;
	 		lcdcursor=0;
			ircursor=0;
			nowmenu=0;
			linemode=false;	
		}
		if(irval>0  && menumode)
		{
			//Serial.println( lcdcursor);
			//Serial.println( nowmenu);
			lcdmenu(lcdcursor, nowmenu );
		}
		
		irrecv.resume(); // Receive the next value
	}

	for(k=0; k<7; k++)
	{
		if(k<6)
		{
			int analogvalue=analogRead(k);
			//Serial.println(analogvalue);
			for(j=0; j<2; j++)
			{
				thispower=powerof(256, j);
				thisbyte=analogvalue/thispower;
	 
				analogvalue=analogvalue-(thisbyte * thispower);
				
				slave_packet[(k*2) + j]=thisbyte;
				//Serial.println(thisbyte + 0);
			
			}
		}
		else//how we send back the last eeprom read -- as the final byte in the slave packet
		{
				slave_packet[12]=lasteepromread;
				slave_packet[13]=lasteepromread;
				for(j=1; j<4; j++)
				{
					slave_packet[13+j]=extrabyte[j-1];
				
				}
		}
	}
	
		//Serial.print("FREEMEMORY ");
		//Serial.print(get_free_memory());
		//Serial.println("");
 
		oldmenumode=menumode;
   delay(300);
} 
 
  
int get_free_memory()
{
  int free_memory;

  if((int)__brkval == 0)
    free_memory = ((int)&free_memory) - ((int)&__bss_end);
  else
    free_memory = ((int)&free_memory) - ((int)__brkval);

  return free_memory;
} 

void outgoing() 
{  
 	byte outsender[3];
	
	//Serial.println("x");
	//Serial.println((int)Wire.receive());
 
 	//Wire.send(slave_packet,18);
	
	 
	
	if(commandbyte==1)//read eeprom
	{
		readvalue=EEPROM.read((unsigned int)address);
		address++;//allow for sequential reads from the EEPROM
		nonlcdmode=false;
 
	}

	else  if (commandbyte==9) //send the slave_packet
	{
		//Serial.println("SLAVE PACKET");
		Wire.send(slave_packet,18);
		nonlcdmode=false;
		return;
	}
	else if (commandbyte==11) //digitalRead
	{
		pinMode(address, INPUT);
		readvalue=digitalRead(address);
		//Serial.println("DIGITAL READ ");
		/*
		outsender[0]=readvalue;
		outsender[1]=readvalue;
		outsender[2]=readvalue;
		Wire.send(outsender,3);
		*/
	 
		nonlcdmode=false;
 
	}

	else if (commandbyte==13) //analogRead
	{
	
		readvalue=analogRead(address);
		//Serial.println("ANALOG READ ");
 
		nonlcdmode=false;
 
	}
	else if (commandbyte==17) //IR value
	{
		readvalue=IRresval % 256;
	}
	else if(commandbyte==15) //RESET!
	{
		void(* resetFunc) (void) = 0;
		resetFunc();
	}
	else if(commandbyte==19) //read menu value
	{
		readvalue=arrDataForMenu[address] ;
	}
	else if(commandbyte==21) //pick up command from menu system
	{
		//need this again...
		Wire.send(commandpacket,18); //doesn't work when commandpacket is char, only byte!!
		ClearCommandPacket();
		nonlcdmode=false;
		return;
	}
	outsender[0]=readvalue;
	outsender[1]=readvalue;
	outsender[2]=readvalue;
 
	Wire.send(outsender,3);
	commandbyte=0;
	return;
} 

void ClearCommandPacket()
{
	byte i;
	for(i=0; i<18; i++)
	{
		commandpacket[i]=0;
	}
}

void setuprequest()
{
	byte b;
	nonlcdmode=true;
	outcount=0;
	commandbyte=99;
 
	//Serial.println("");
	while(Wire.available() )
	{
		b=Wire.receive();
		

		//order: command byte, addressbyte high, addressbyte low, value
		if(outcount==0)
		{
			commandbyte=b;
			if(commandbyte==9)
			{
	
			}
		}
		else if(outcount==1  && commandbyte!=9)
		{

			high=(byte)b;
		}
		else if(outcount==2  && commandbyte!=9)
		{
			low=(byte)b;
		}
		else if(outcount==3  && commandbyte!=9)
		{
			readvalue=b;
		}
		if(commandbyte!=9)
		{
			address=(unsigned int)high*256 + low;
		}
		if(1==2)
		{
			Serial.print("^");
			Serial.print((char)high);
			Serial.print(" ");
			Serial.print((char)low);
			Serial.print(" ");
			Serial.print(address);
			Serial.println("");
		}
		outcount++;
	}
	
	if (commandbyte==2) //write eeprom
	{
		//Serial.print("EEPROM WRITE ");
		Serial.print(address);
		//Serial.print(":");
		Serial.print((int) readvalue);
		Serial.println("");
		EEPROM.write(address, readvalue);
		nonlcdmode=false;
 
		//deliverable[0]=77;
		//Wire.send(deliverable,1);
	
	}
	else if (commandbyte==10) //digitalWrite
	{
	
		pinMode(address, OUTPUT);
		digitalWrite(address, readvalue);
		//Serial.println("DIGITAL WRITE ");
		nonlcdmode=false;
	 
		
		//printstuff(1) ;
	}
	else if (commandbyte==14) //analogWrite
	{
	
		analogWrite(address, readvalue);
		//Serial.println("ANALOG WRITE ");
		nonlcdmode=false;
		
		//printstuff(1) ;
	} 
	else if (commandbyte==16) //setDefaultMenuValues
	{
	
		arrDataForMenu[address]= readvalue;
 
		nonlcdmode=false;
		canmenu=true;
		Serial.println("can menu!! ");
		Serial.println("11:");
		Serial.println(arrDataForMenu[11]);
		//printstuff(1) ;
	} 
	nonlcdmode=false;
	return;
}

 
void printstuff(int number) //display delimited data from Master on the LCD
{ 

	//Serial.println(" ");
	//Serial.println(number);
	if(number<1)
	{
		return; //wrong kind of access!
	}
	char * thisitem[40];
	int count=0;
	int fieldcount=0;
	int itemcount=0;
	char b;
	char fielddelimiter='|';
	char columndelimiter='^';
	int thistemp;
	int i;
	int j;
	int thisint;
	int charpos;
	
	byte extrabytecounter=0;
 	byte k;
	outcount=0;
	bool bwlDoingCharactersNow=false;
	charpos=0;
 
 	thisint=0;
	
	while(Wire.available()>0  && !nonlcdmode) 
	{
		
		b=Wire.receive();
 
		if(b=='!' && count==0 || nonlcdmode)
		{
			setuprequest();
			return;

		}
		//Serial.print('%');
		//Serial.println(b);
		if(b==fielddelimiter)
		{
			//Serial.print("$");
			if(fieldcount==0)
			{
				//Serial.print("#");
				xpos[itemcount]=thisint;
				thisint=0;
				charpos=0;
			}
			else if(fieldcount==1)
			{
				ypos[itemcount]=thisint;
				thisint=0;
				charpos=0;
			}
			//the following allow the master to pass in a list of up to four bytes to be stored consecutively
			else if(fieldcount==2)
			{
				extrabyte[0]==thisint;
				extrabytecounter++;
			}
			else if(fieldcount==3)
			{
				extrabyte[1]==thisint;
				extrabytecounter++;
			}
			else if(fieldcount==4)
			{
				extrabyte[2]==thisint;
				extrabytecounter++;
			}
			fieldcount++;
		}
		else if (b==columndelimiter)
		{
			//Serial.print("@");
			itemcount++;
			thisint=0;
			fieldcount=0;
			charpos=0;
			bwlDoingCharactersNow=false;
		}
		else
		{
			if(fieldcount<2  || (b>47  && b<58  && (charpos==0  || !bwlDoingCharactersNow)))
			{
				thisint=thisint*10+(b-48);
			}
			else
			{
				charblock[charpos][itemcount]=b;
				charblock[charpos+1][itemcount]=0;
				charpos++;
				thisint=0;
				bwlDoingCharactersNow=true;
			}
			
		}
		//Serial.print(thisbyte[count]);
		//Serial.print("*");
 		
 
		count++;
		//number--;
	}
 	if(count>0  && !nonlcdmode)
	{
		//Serial.println(" ");
		//Serial.println(itemcount);
		//Serial.print(xpos[i]);
		//Serial.print(" ");
		//Serial.print(ypos[i]);
		//Serial.println(charblock[j]);
		//Serial.println("");
		for(i=0; i<=itemcount; i++)
		{
			if(1==1 && !menumode)
			{
				//Serial.print("x:");
				//Serial.print(xpos[i]);
				//Serial.print(" y:");
				///Serial.print(ypos[i]);
				//Serial.println(" ");
				if(xpos[i]<100)
				{
					
					lcd.setCursor(xpos[i], ypos[i]);
					for(j=0; j<20; j++)
					{
						if(charblock[j][i]!=0)
						{
							lcd.print(charblock[j][i]);
							//Serial.print(charblock[j][i]);
						}
					}
				}
				else if (xpos[i]<1000)//if we're well outside the range of the LCD (above 100 and below 1000), then use this command to set the pin at xpos-100 with the value of ypos
				{
					
					if( ypos[i]<256)
					{
						Serial.println("found a pinmoder");
						 pinMode(xpos[i]-100, OUTPUT);
						 if(ypos[i]>0)
						 {
							 digitalWrite(xpos[i]-100, 1);
						 }
						 else
						 {
						 	digitalWrite(xpos[i]-100,0);
						 
						 	analogWrite(xpos[i]-100, ypos[i]);
						 }
					 }
					 else
					 {
					 	if(ypos[i]==256)
						{
							Serial.println("found a digital pin reader");
							pinMode(xpos[i]-100, INPUT);
							lastpinread=digitalRead(xpos[i]-100);
						}
						else
						{
							Serial.println("found an analog pin reader");
							lastpinread=analogRead(xpos[i]-100);
						}
					 	
					 	
					 }
				}
				else //use this to write in the slave EEPROM at the address of xpos-1000.
				{
					
					if( ypos[i]<256)
					{
						Serial.println("found an eeprom writer");
						EEPROM.write(xpos[i]-1000, ypos[i]);
						if(extrabytecounter>0)
						{
							
							//write out any more bytes that were sent, up to three
							for(k=0; k<extrabytecounter; k++)
							{
								EEPROM.write(k + 1+ (xpos[i]-1000), extrabyte[k]);
							}
						}
						lasteepromread=ypos[i];
					}
					else if(1==2)
					{
						Serial.println("found an eeprom reader");
						lasteepromread=EEPROM.read(xpos[i]-1000);
						for(k=0; k<3; k++)
						{
							//Serial.print(k+xpos[i]+1-1000);
							//Serial.print(": ");
							extrabyte[k]=EEPROM.read(k+1+xpos[i]-1000);
							//Serial.print((int)extrabyte[k]);
							//Serial.print(", ");
						}
						Serial.println("");
					}
				}
				//Serial.println(" ");
			}
			//Serial.println(" ");
		}
	
	}
 
	if(1==2)
	{
		Serial.print(temp1);
	 	Serial.print(" ");
		Serial.print(temp2);
		Serial.print(" ");
		Serial.print(temp3);
	 	Serial.print(" ");
		Serial.print(temp4);
		Serial.print(" ");
		Serial.println(" ");
	}
	//lcd.print(thisbyte + 0);
   
} 
 

long powerof(long intin, byte powerin)
//raises intin to the power of powerin
//not using "pow" saves me a lot of Flash memory
{
	long outdata=1;
	byte k;
	for (k = 1; k <powerin+1; k++)
	{
		outdata=outdata*intin;
	}
	return outdata;
}

int ircodelookup(unsigned int inval)
//this will vary for different IR remotes but i'm setting it up for the one i happen to have
{
 
	unsigned int i;
	unsigned int outval=0;
	//Serial.println(sizeof(integers)/sizeof(int));


	for(i=0; i< (sizeof(commands)/sizeof(int)); i++)
	{
		if((unsigned int)commands[i]==(unsigned int)inval)
		{
			outval=i;
		}
		
	}
	
	for(i=0; i<(sizeof(integers)/sizeof(int)); i++)
	{
		if((unsigned int)integers[i]==(unsigned int)inval)
		{
			if(i==10)
			{
				outval=58;
			}
			else if (i==11)
			{
				outval=45;
			}
			else
			{
				outval = i + 48;
			}
		}

		
	}
	return outval;

}

void printchars(byte numberofchars, char charin, byte mode)
{
	byte i;
	for(i=0; i<numberofchars; i++)
	{
		if(mode==0)
		{
			Serial.print(charin);
		}
		else if(mode==1)
		{
			Wire.send(charin);
		}
		else if(mode==2)
		{
			lcd.print(charin);
		}
	}
}

void lcdmenu(int placement, int level )
{ 
	
	int i=0;
	int locallow, localhigh, intmaxatonce;
	byte thisy=0;
	char pre=' ';
	char * stufftoprint;
	intmaxatonce=4;
	
	ClearLCD();
 	if(level<3)
 	{
		if(placement<intmaxatonce)
		{
			locallow=0;
			localhigh=locallow+intmaxatonce;
		}
		else
		{
			locallow=	placement;
			localhigh=locallow+intmaxatonce;
		}
		if(localhigh>lcdtop)
		{
			
			localhigh=lcdtop;	
			locallow=	localhigh-intmaxatonce;	
		}
	 	
		for(i=locallow; i<localhigh; i++)
		{
			if(i==placement)
			{
				pre='+';
			}
			else
			{
				pre=' ';	
			}
			Serial.print(pre);
			stufftoprint=(char *)menuitem[level][ i];
				
			if(stufftoprint[0]!='*')
			{
				Serial.println(stufftoprint);
				lcd.setCursor(0, thisy);
				lcd.print(pre);
				lcd.print(stufftoprint);
				lcd.print("");
				thisy++;
			}
			else
			{
				break;
			}
		
		}
	}
 	else
 	{
 		if(level==4)
 		{
 			displaydelimiteddata(level);
 			DisplayCursor(level);
 		}
 		else if(level==3)
 		{
 			displaydelimiteddata(level);
 			DisplayCursor(level);
 		}
 		else if(level>4) //5 is season!
 		{
 			displaynondelimiteddata(level);
 			DisplayCursor(level);
 		}
 	}
}


byte arraylimit(const char ** arrIn) //looks for the end of an array as tagged by "*"
{
	byte i=0;
	for(i=0; i<200; i++)
	{
		if(arrIn[i][0]=='*')
		{
			return i;
			break;
			
		}
	}
	return 200;
}

 
void populatedefaultarray() //not really even necessary except to give default values should the master not send them
{
	byte i,j;
	byte thiscounter=0;
	byte charinthis=0;
	char thischar;
	byte thisseries[3];
	
	for(i=0; i<70; i++)
	{
		//Serial.println((int) i);
		thischar=irinternaldata[i];
		if(thischar==',')
		{
			for(j=0; j<charinthis; j++)
			{
				//Serial.print((int) j);
				//Serial.print(" ");
				//Serial.print((int)thisseries[j]);
				//Serial.print(" ");
				//Serial.print((charinthis-1)-j);
				//Serial.print(" ");
				//Serial.print((int) powerof(10,(charinthis-1)-j));
				//Serial.println(" ");
				arrDataForMenu[thiscounter]+=(int) thisseries[j] * powerof(10,(charinthis-1)-j);
			}
			thiscounter++;
			charinthis=0;
			
		}
		else
		{
			thisseries[charinthis]=thischar-48;
			charinthis++;
		}
	
	
	}

}

unsigned long powerof(int intin, byte powerin)
//raises intin to the power of powerin
//not using "pow" saves me a lot of Flash memory
{
  unsigned long outdata=1;
  byte k;
  for (k = 1; k <powerin+1; k++)
  {
    outdata=outdata*intin;
  }
  return outdata;
}

void pwlvin(byte inval)
{
	if((int)inval<10)
	{
		Serial.print("0");
 
		lcd.print("0");
  
	}
	Serial.print((int)inval);
	lcd.print((int)inval);
}

void pwlv3in(int inval)
{
	if((int)inval<100)
	{
		Serial.print("0");
		lcd.print("0");
	}
	if((int)inval<10)
	{
		Serial.print("0");
		lcd.print("0");
	}
	Serial.print((int)inval);
	lcd.print((int)inval);
}

void displaydelimiteddata(  byte level)
{
	byte thisy=0;
	ClearLCD();
	Serial.println(menuitem[0][level-1]);
	
 

	lcd.setCursor(0, thisy);
	lcd.print(menuitem[0][level-1]);
	thisy++;
				
	char separator='-';
	if(level==4)
	{
		separator=':';	
	}
	lcd.setCursor(0, thisy);
	for(byte i=0; i<3; i++)
	{
		Serial.print(legend[level-3][i]);
		

		
		lcd.print(legend[level-3][i]);
	 
		
		if(i<2)
		{
			Serial.print(separator);
			lcd.print(separator);
		}
	}
	Serial.println("");
	thisy++;

	lcd.setCursor(0, thisy);
	for(byte i=0; i<3; i++)
	{
		pwlvin(arrDataForMenu[dataplace[level-1][i]]);
		if(i<2)
		{
			Serial.print(separator);
			lcd.print(separator);
		}
	}
	Serial.println("");
	thisy++;

	lcd.setCursor(0, thisy);
	
}

void displaynondelimiteddata(byte level)
{
	byte thisy=0;
	ClearLCD();
	lcd.setCursor(0, thisy);
	lcd.print(menuitem[0][level-1]);
	thisy++;
	
	Serial.println(menuitem[0][level-1]);
	

	lcd.setCursor(0, thisy);
	pwlv3in(arrDataForMenu[level+3]);
	 

	//lcd.setCursor(0, thisy);
	//printchars(20, ' ', 2);
	//lcd.setCursor(0, thisy);
	if(level<8)
	{
		lcd.print(" deg. F");
		Serial.print(" deg. F");
	}
	else
	{
		lcd.print(" deciseconds");
		Serial.print(" deciseconds");
	}

	Serial.println("");
	thisy++;

	lcd.setCursor(0, thisy);
}

void ClearLCD()
{
	byte i;
	for(i=0; i<4; i++)
	{
		lcd.setCursor(0, i);
		printchars(20, ' ', 2);
	}
}

void IRchangeInternalData(byte level, byte irval)
{
	//only worry about arrDataForMenu, not irinternaldata!
	byte i;
 	int arrSeq[3] ;
	byte thispower;
 	byte otherpart;
	int thisval;
	//next line refers to displaydelimiteddata order:
	//SECONDS,MINUTES,HOURS,DAYOFWEEK,DAY,MONTH,YEAR,MAXHOTWATER,MINSUMMER,MINWINTER,CYCLETIME
	//Serial.println((int)level);
	if(level==3  || level==4)
	{
		for(i=0; i<6; i++)
		{
			arrSeq[(int)i/2]=dataplace[level-1][i/2];
			if((int)i % 2==1)
			{
				thispower=0;
			}
			else 
			{
				thispower=1;
			}
			if((int)i==(int)ircursor)
			{
				thisval=arrDataForMenu[arrSeq[(int)i/2]];
				arrDataForMenu[arrSeq[(int)i/2]]=   SwapInDigit(irval-48, thispower, thisval)  ; 
			} 
		}
	}
	else if (level>4)
	{
		arrSeq[0]=level+3; //where a temperature cutoff/time value falls in the internal list
		for(i=0; i<3; i++)
		{
			thispower=2-i;	
			thisval=arrDataForMenu[arrSeq[0]];
			//Serial.println((int)thisval);
			if((int)i==(int)ircursor)
			{
				
				arrDataForMenu[arrSeq[0]]=   SwapInDigit(irval-48, thispower, thisval)  ; 
			}
		}
		
	}
	else
	{
		arrSeq[0]=dataplace[level-1][0];
		//Serial.print("valontherebound:");
		//Serial.println(arrSeq[0]);
		arrDataForMenu[arrSeq[0]]=irval;
	}
}

int SwapInDigit(byte inval, byte distancefromones, int originalvalue)
//allows me to take an integer and swap in a new digit to replace an existing one
//works for integers up to ten thousand
{
	int out=0;
	int scanval;
	for(byte i=6; i<255; i--)//weird how i had to do that freaky 255 test
	{
	
		scanval=originalvalue/powerof(10, i);
		if(distancefromones==i  )
		{
			out=out+ inval*powerof(10, i);
		}
		else
		{
			out=out+ scanval*powerof(10, i);
		}
		originalvalue=originalvalue-scanval* powerof(10, i);
	}
	return out;
	
}

void DisplayCursor(byte level)
{
	//Serial.println("");
	for(byte i=0; i<ircursor; i++)
	{
		Serial.print("-");
		lcd.print("-");
		if((i+1) % 2==0  && i>0  && level>2  && level<5)
		{
			Serial.print("-");
			lcd.print("-");
		}
	}
	Serial.println("^");
	lcd.print("^");
}
